import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { Disposable, MutableDisposable, DisposableStore } from 'vscode/vscode/vs/base/common/lifecycle';
import { isWeb } from 'vscode/vscode/vs/base/common/platform';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { MenuId, MenuRegistry, registerAction2, Action2 } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IMenuService } from 'vscode/vscode/vs/platform/actions/common/actions.service';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey.service';
import { IUserDataProfilesService } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile.service';
import { LifecyclePhase } from 'vscode/vscode/vs/workbench/services/lifecycle/common/lifecycle';
import { ILifecycleService } from 'vscode/vscode/vs/workbench/services/lifecycle/common/lifecycle.service';
import { CURRENT_PROFILE_CONTEXT, PROFILES_ENABLEMENT_CONTEXT, IS_CURRENT_PROFILE_TRANSIENT_CONTEXT, HAS_PROFILES_CONTEXT, ProfilesMenu, PROFILES_CATEGORY, IS_PROFILE_EXPORT_IN_PROGRESS_CONTEXT, IS_PROFILE_IMPORT_IN_PROGRESS_CONTEXT, PROFILE_FILTER, PROFILES_TITLE } from 'vscode/vscode/vs/workbench/services/userDataProfile/common/userDataProfile';
import { IUserDataProfileImportExportService, IUserDataProfileService, IUserDataProfileManagementService } from 'vscode/vscode/vs/workbench/services/userDataProfile/common/userDataProfile.service';
import { IQuickInputService } from 'vscode/vscode/vs/platform/quickinput/common/quickInput.service';
import { INotificationService } from 'vscode/vscode/vs/platform/notification/common/notification.service';
import { IFileDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry.service';
import { WorkbenchState } from 'vscode/vscode/vs/platform/workspace/common/workspace';
import { IWorkspaceContextService } from 'vscode/vscode/vs/platform/workspace/common/workspace.service';
import { IWorkspaceTagsService } from 'vscode/vscode/vs/workbench/contrib/tags/common/workspaceTags.service';
import { getErrorMessage } from 'vscode/vscode/vs/base/common/errors';
import { Categories } from 'vscode/vscode/vs/platform/action/common/actionCommonCategories';
import { IOpenerService } from 'vscode/vscode/vs/platform/opener/common/opener.service';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { EditorPaneDescriptor } from 'vscode/vscode/vs/workbench/browser/editor';
import { EditorExtensions } from 'vscode/vscode/vs/workbench/common/editor';
import { UserDataProfilesEditorInput, UserDataProfilesEditor, UserDataProfilesEditorInputSerializer } from './userDataProfilesEditor.js';
import { SyncDescriptor } from 'vscode/vscode/vs/platform/instantiation/common/descriptors';
import { IEditorGroupsService } from 'vscode/vscode/vs/workbench/services/editor/common/editorGroupsService.service';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IHostService } from 'vscode/vscode/vs/workbench/services/host/browser/host.service';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { Extensions, ConfigurationScope } from 'vscode/vscode/vs/platform/configuration/common/configurationRegistry';
import { workbenchConfigurationNodeBase } from 'vscode/vscode/vs/workbench/common/configuration';
import { IProductService } from 'vscode/vscode/vs/platform/product/common/productService.service';

const _moduleId = "vs/workbench/contrib/userDataProfile/browser/userDataProfile";
const OpenProfileMenu = ( (new MenuId('OpenProfile')));
const CONFIG_ENABLE_NEW_PROFILES_UI = 'workbench.experimental.enableNewProfilesUI';
const CONTEXT_ENABLE_NEW_PROFILES_UI = ( (ContextKeyExpr.equals('config.workbench.experimental.enableNewProfilesUI', true)));
let UserDataProfilesWorkbenchContribution = class UserDataProfilesWorkbenchContribution extends Disposable {
    static { this.ID = 'workbench.contrib.userDataProfiles'; }
    constructor(userDataProfileService, userDataProfilesService, userDataProfileManagementService, userDataProfileImportExportService, telemetryService, workspaceContextService, workspaceTagsService, contextKeyService, configurationService, editorGroupsService, instantiationService, productService, lifecycleService) {
        super();
        this.userDataProfileService = userDataProfileService;
        this.userDataProfilesService = userDataProfilesService;
        this.userDataProfileManagementService = userDataProfileManagementService;
        this.userDataProfileImportExportService = userDataProfileImportExportService;
        this.telemetryService = telemetryService;
        this.workspaceContextService = workspaceContextService;
        this.workspaceTagsService = workspaceTagsService;
        this.configurationService = configurationService;
        this.editorGroupsService = editorGroupsService;
        this.instantiationService = instantiationService;
        this.productService = productService;
        this.lifecycleService = lifecycleService;
        this.startTime = Date.now();
        this.profilesDisposable = this._register(( (new MutableDisposable())));
        this.currentprofileActionsDisposable = this._register(( (new MutableDisposable())));
        this.currentProfileContext = CURRENT_PROFILE_CONTEXT.bindTo(contextKeyService);
        PROFILES_ENABLEMENT_CONTEXT.bindTo(contextKeyService).set(this.userDataProfilesService.isEnabled());
        this.isCurrentProfileTransientContext = IS_CURRENT_PROFILE_TRANSIENT_CONTEXT.bindTo(contextKeyService);
        this.currentProfileContext.set(this.userDataProfileService.currentProfile.id);
        this.isCurrentProfileTransientContext.set(!!this.userDataProfileService.currentProfile.isTransient);
        this._register(this.userDataProfileService.onDidChangeCurrentProfile(e => {
            this.currentProfileContext.set(this.userDataProfileService.currentProfile.id);
            this.isCurrentProfileTransientContext.set(!!this.userDataProfileService.currentProfile.isTransient);
        }));
        this.hasProfilesContext = HAS_PROFILES_CONTEXT.bindTo(contextKeyService);
        this.hasProfilesContext.set(this.userDataProfilesService.profiles.length > 1);
        this._register(this.userDataProfilesService.onDidChangeProfiles(e => this.hasProfilesContext.set(this.userDataProfilesService.profiles.length > 1)));
        this.registerConfiguration();
        this.registerEditor();
        this.registerActions();
        if (isWeb) {
            lifecycleService.when(LifecyclePhase.Eventually).then(() => userDataProfilesService.cleanUp());
        }
        this.reportWorkspaceProfileInfo();
    }
    openProfilesEditor() {
        return this.editorGroupsService.activeGroup.openEditor(( (new UserDataProfilesEditorInput(this.instantiationService))));
    }
    isNewProfilesUIEnabled() {
        return this.configurationService.getValue(CONFIG_ENABLE_NEW_PROFILES_UI) === true;
    }
    registerConfiguration() {
        ( (Registry.as(Extensions.Configuration)))
            .registerConfiguration({
            ...workbenchConfigurationNodeBase,
            properties: {
                [CONFIG_ENABLE_NEW_PROFILES_UI]: {
                    type: 'boolean',
                    description: ( localizeWithPath(_moduleId, 0, "Enables the new profiles UI.")),
                    default: this.productService.quality !== 'stable',
                    scope: ConfigurationScope.APPLICATION,
                }
            }
        });
    }
    registerEditor() {
        ( (Registry.as(EditorExtensions.EditorPane))).registerEditorPane(EditorPaneDescriptor.create(UserDataProfilesEditor, UserDataProfilesEditor.ID, ( localizeWithPath(_moduleId, 1, "Profiles Editor"))), [
            ( (new SyncDescriptor(UserDataProfilesEditorInput)))
        ]);
        ( (Registry.as(EditorExtensions.EditorFactory))).registerEditorSerializer(UserDataProfilesEditorInput.ID, UserDataProfilesEditorInputSerializer);
    }
    registerActions() {
        this.registerProfileSubMenu();
        this._register(this.registerManageProfilesAction());
        this._register(this.registerSwitchProfileAction());
        this.registerOpenProfileSubMenu();
        this.registerProfilesActions();
        this._register(this.userDataProfilesService.onDidChangeProfiles(() => this.registerProfilesActions()));
        this.registerCurrentProfilesActions();
        this._register(this.userDataProfileService.onDidChangeCurrentProfile(() => this.registerCurrentProfilesActions()));
        this.registerCreateFromCurrentProfileAction();
        this.registerCreateProfileAction();
        this.registerDeleteProfileAction();
        this.registerHelpAction();
    }
    registerProfileSubMenu() {
        const getProfilesTitle = () => {
            return ( localizeWithPath(
                _moduleId,
                2,
                "Profile ({0})",
                this.userDataProfileService.currentProfile.name
            ));
        };
        const when = ( (ContextKeyExpr.or(
             (CONTEXT_ENABLE_NEW_PROFILES_UI.negate()),
            HAS_PROFILES_CONTEXT
        )));
        MenuRegistry.appendMenuItem(MenuId.GlobalActivity, {
            get title() {
                return getProfilesTitle();
            },
            submenu: ProfilesMenu,
            group: '2_configuration',
            order: 1,
            when,
        });
        MenuRegistry.appendMenuItem(MenuId.MenubarPreferencesMenu, {
            get title() {
                return getProfilesTitle();
            },
            submenu: ProfilesMenu,
            group: '2_configuration',
            order: 1,
            when,
        });
    }
    registerOpenProfileSubMenu() {
        MenuRegistry.appendMenuItem(MenuId.MenubarFileMenu, {
            title: ( localizeWithPath(_moduleId, 3, "New Window with Profile")),
            submenu: OpenProfileMenu,
            group: '1_new',
            order: 4,
            when: HAS_PROFILES_CONTEXT,
        });
    }
    registerProfilesActions() {
        this.profilesDisposable.value = ( (new DisposableStore()));
        for (const profile of this.userDataProfilesService.profiles) {
            if (!profile.isTransient) {
                this.profilesDisposable.value.add(this.registerProfileEntryAction(profile));
                this.profilesDisposable.value.add(this.registerNewWindowAction(profile));
            }
        }
    }
    registerProfileEntryAction(profile) {
        const that = this;
        return registerAction2(class ProfileEntryAction extends Action2 {
            constructor() {
                super({
                    id: `workbench.profiles.actions.profileEntry.${profile.id}`,
                    title: profile.name,
                    toggled: ( (ContextKeyExpr.equals(CURRENT_PROFILE_CONTEXT.key, profile.id))),
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '0_profiles',
                            when: PROFILES_ENABLEMENT_CONTEXT,
                        }
                    ]
                });
            }
            async run(accessor) {
                if (that.userDataProfileService.currentProfile.id !== profile.id) {
                    if (profile.isDefault && Date.now() - that.startTime < ((1000 * 20) )) {
                        that.telemetryService.publicLog2('profiles:newwindowprofile', {
                            emptyWindow: that.workspaceContextService.getWorkbenchState() === WorkbenchState.EMPTY
                        });
                    }
                    return that.userDataProfileManagementService.switchProfile(profile);
                }
            }
        });
    }
    registerNewWindowAction(profile) {
        const disposables = ( (new DisposableStore()));
        const id = `workbench.action.openProfile.${profile.name.toLowerCase().replace('/\s+/', '_')}`;
        disposables.add(registerAction2(class NewWindowAction extends Action2 {
            constructor() {
                super({
                    id,
                    title: ( localize2WithPath(_moduleId, 4, "{0}", profile.name)),
                    menu: {
                        id: OpenProfileMenu,
                        when: HAS_PROFILES_CONTEXT
                    }
                });
            }
            run(accessor) {
                const hostService = accessor.get(IHostService);
                return hostService.openWindow({ remoteAuthority: null, forceProfile: profile.name });
            }
        }));
        disposables.add(MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
            command: {
                id,
                category: PROFILES_CATEGORY,
                title: ( localize2WithPath(_moduleId, 5, "Open {0} Profile", profile.name)),
                precondition: HAS_PROFILES_CONTEXT
            },
        }));
        return disposables;
    }
    registerSwitchProfileAction() {
        return registerAction2(class SwitchProfileAction extends Action2 {
            constructor() {
                super({
                    id: `workbench.profiles.actions.switchProfile`,
                    title: ( localize2WithPath(_moduleId, 6, 'Switch Profile...')),
                    category: PROFILES_CATEGORY,
                    f1: true,
                    precondition: PROFILES_ENABLEMENT_CONTEXT,
                });
            }
            async run(accessor) {
                const quickInputService = accessor.get(IQuickInputService);
                const menuService = accessor.get(IMenuService);
                const menu = menuService.createMenu(ProfilesMenu, accessor.get(IContextKeyService));
                const actions = menu.getActions().find(([group]) => group === '0_profiles')?.[1] ?? [];
                try {
                    const result = await quickInputService.pick(( (actions.map(action => ({
                        action,
                        label: action.checked ? `$(check) ${action.label}` : action.label,
                    })))), {
                        placeHolder: ( localizeWithPath(_moduleId, 7, "Select Profile"))
                    });
                    await result?.action.run();
                }
                finally {
                    menu.dispose();
                }
            }
        });
    }
    registerCurrentProfilesActions() {
        this.currentprofileActionsDisposable.value = ( (new DisposableStore()));
        this.currentprofileActionsDisposable.value.add(this.registerEditCurrentProfileAction());
        this.currentprofileActionsDisposable.value.add(this.registerShowCurrentProfileContentsAction());
        this.currentprofileActionsDisposable.value.add(this.registerExportCurrentProfileAction());
        this.currentprofileActionsDisposable.value.add(this.registerImportProfileAction());
    }
    registerManageProfilesAction() {
        const disposables = ( (new DisposableStore()));
        disposables.add(registerAction2(class ManageProfilesAction extends Action2 {
            constructor() {
                super({
                    id: `workbench.profiles.actions.manageProfiles`,
                    title: {
                        ...( localize2WithPath(_moduleId, 8, "Profiles")),
                        mnemonicTitle: ( localizeWithPath(_moduleId, 9, "&&Profiles")),
                    },
                    menu: [
                        {
                            id: MenuId.GlobalActivity,
                            group: '2_configuration',
                            order: 1,
                            when: CONTEXT_ENABLE_NEW_PROFILES_UI,
                        },
                        {
                            id: MenuId.MenubarPreferencesMenu,
                            group: '2_configuration',
                            order: 1,
                            when: CONTEXT_ENABLE_NEW_PROFILES_UI,
                        },
                    ]
                });
            }
            run(accessor) {
                const editorGroupsService = accessor.get(IEditorGroupsService);
                const instantiationService = accessor.get(IInstantiationService);
                return editorGroupsService.activeGroup.openEditor(( (new UserDataProfilesEditorInput(instantiationService))));
            }
        }));
        disposables.add(MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
            command: {
                id: 'workbench.profiles.actions.manageProfiles',
                category: Categories.Preferences,
                title: ( localize2WithPath(_moduleId, 10, "Open Profiles (UI)")),
                precondition: CONTEXT_ENABLE_NEW_PROFILES_UI,
            },
        }));
        return disposables;
    }
    registerEditCurrentProfileAction() {
        const that = this;
        return registerAction2(class RenameCurrentProfileAction extends Action2 {
            constructor() {
                const precondition = ( (ContextKeyExpr.and( (ContextKeyExpr.notEquals(
                    CURRENT_PROFILE_CONTEXT.key,
                    that.userDataProfilesService.defaultProfile.id
                )),  (IS_CURRENT_PROFILE_TRANSIENT_CONTEXT.toNegated()))));
                super({
                    id: `workbench.profiles.actions.editCurrentProfile`,
                    title: ( localize2WithPath(_moduleId, 11, "Edit Profile...")),
                    precondition,
                    f1: true,
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '2_manage_current',
                            when: ( (ContextKeyExpr.and(precondition,  (CONTEXT_ENABLE_NEW_PROFILES_UI.negate())))),
                            order: 2
                        }
                    ]
                });
            }
            run(accessor) {
                if (that.isNewProfilesUIEnabled()) {
                    return that.openProfilesEditor();
                }
                else {
                    return that.userDataProfileImportExportService.editProfile(that.userDataProfileService.currentProfile);
                }
            }
        });
    }
    registerShowCurrentProfileContentsAction() {
        const id = 'workbench.profiles.actions.showProfileContents';
        return registerAction2(class ShowProfileContentsAction extends Action2 {
            constructor() {
                super({
                    id,
                    title: ( localize2WithPath(_moduleId, 12, "Show Profile Contents")),
                    category: PROFILES_CATEGORY,
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '2_manage_current',
                            order: 3,
                            when: ( (CONTEXT_ENABLE_NEW_PROFILES_UI.negate()))
                        }
                    ]
                });
            }
            async run(accessor) {
                const userDataProfileImportExportService = accessor.get(IUserDataProfileImportExportService);
                return userDataProfileImportExportService.showProfileContents();
            }
        });
    }
    registerExportCurrentProfileAction() {
        const that = this;
        const disposables = ( (new DisposableStore()));
        const id = 'workbench.profiles.actions.exportProfile';
        disposables.add(registerAction2(class ExportProfileAction extends Action2 {
            constructor() {
                super({
                    id,
                    title: ( localize2WithPath(_moduleId, 13, "Export Profile...")),
                    category: PROFILES_CATEGORY,
                    precondition: ( (IS_PROFILE_EXPORT_IN_PROGRESS_CONTEXT.toNegated())),
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '4_import_export_profiles',
                            order: 1,
                            when: ( (CONTEXT_ENABLE_NEW_PROFILES_UI.negate())),
                        }, {
                            id: MenuId.CommandPalette
                        }
                    ]
                });
            }
            async run(accessor) {
                if (that.isNewProfilesUIEnabled()) {
                    return that.openProfilesEditor();
                }
                else {
                    return that.userDataProfileImportExportService.exportProfile2();
                }
            }
        }));
        disposables.add(MenuRegistry.appendMenuItem(MenuId.MenubarShare, {
            command: {
                id,
                title: ( localize2WithPath(
                    _moduleId,
                    14,
                    "Export Profile ({0})...",
                    that.userDataProfileService.currentProfile.name
                )),
                precondition: PROFILES_ENABLEMENT_CONTEXT,
            },
        }));
        return disposables;
    }
    registerImportProfileAction() {
        const disposables = ( (new DisposableStore()));
        const id = 'workbench.profiles.actions.importProfile';
        const that = this;
        disposables.add(registerAction2(class ImportProfileAction extends Action2 {
            constructor() {
                super({
                    id,
                    title: ( localize2WithPath(_moduleId, 15, "Import Profile...")),
                    category: PROFILES_CATEGORY,
                    precondition: ( (IS_PROFILE_IMPORT_IN_PROGRESS_CONTEXT.toNegated())),
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '4_import_export_profiles',
                            when: ( (ContextKeyExpr.and(
                                PROFILES_ENABLEMENT_CONTEXT,
                                 (CONTEXT_ENABLE_NEW_PROFILES_UI.negate())
                            ))),
                            order: 2
                        }, {
                            id: MenuId.CommandPalette,
                            when: PROFILES_ENABLEMENT_CONTEXT,
                        }
                    ]
                });
            }
            async run(accessor) {
                const fileDialogService = accessor.get(IFileDialogService);
                const quickInputService = accessor.get(IQuickInputService);
                const userDataProfileImportExportService = accessor.get(IUserDataProfileImportExportService);
                const notificationService = accessor.get(INotificationService);
                const disposables = ( (new DisposableStore()));
                const quickPick = disposables.add(quickInputService.createQuickPick());
                const profileTemplateQuickPickItems = await that.getProfileTemplatesQuickPickItems();
                const updateQuickPickItems = (value) => {
                    const quickPickItems = [];
                    if (value) {
                        quickPickItems.push({ label: quickPick.value, description: ( localizeWithPath(_moduleId, 16, "Import from URL")) });
                    }
                    quickPickItems.push({ label: ( localizeWithPath(_moduleId, 17, "Select File...")) });
                    if (profileTemplateQuickPickItems.length) {
                        quickPickItems.push({
                            type: 'separator',
                            label: ( localizeWithPath(_moduleId, 18, "Profile Templates"))
                        }, ...profileTemplateQuickPickItems);
                    }
                    quickPick.items = quickPickItems;
                };
                quickPick.title = ( localizeWithPath(_moduleId, 19, "Import from Profile Template..."));
                quickPick.placeholder = ( localizeWithPath(_moduleId, 20, "Provide Profile Template URL"));
                quickPick.ignoreFocusOut = true;
                disposables.add(quickPick.onDidChangeValue(updateQuickPickItems));
                updateQuickPickItems();
                quickPick.matchOnLabel = false;
                quickPick.matchOnDescription = false;
                disposables.add(quickPick.onDidAccept(async () => {
                    quickPick.hide();
                    const selectedItem = quickPick.selectedItems[0];
                    if (!selectedItem) {
                        return;
                    }
                    try {
                        if (selectedItem.url) {
                            return await that.userDataProfileImportExportService.createProfile(( (URI.parse(selectedItem.url))));
                        }
                        const profile = selectedItem.label === quickPick.value ? ( (URI.parse(quickPick.value))) : await this.getProfileUriFromFileSystem(fileDialogService);
                        if (profile) {
                            await userDataProfileImportExportService.importProfile(profile);
                        }
                    }
                    catch (error) {
                        notificationService.error(( localizeWithPath(_moduleId, 21, "Error while creating profile: {0}", getErrorMessage(error))));
                    }
                }));
                disposables.add(quickPick.onDidHide(() => disposables.dispose()));
                quickPick.show();
            }
            async getProfileUriFromFileSystem(fileDialogService) {
                const profileLocation = await fileDialogService.showOpenDialog({
                    canSelectFolders: false,
                    canSelectFiles: true,
                    canSelectMany: false,
                    filters: PROFILE_FILTER,
                    title: ( localizeWithPath(_moduleId, 22, "Select Profile Template File")),
                });
                if (!profileLocation) {
                    return null;
                }
                return profileLocation[0];
            }
        }));
        disposables.add(MenuRegistry.appendMenuItem(MenuId.MenubarShare, {
            command: {
                id,
                title: ( localize2WithPath(_moduleId, 23, "Import Profile...")),
                precondition: PROFILES_ENABLEMENT_CONTEXT,
            },
        }));
        return disposables;
    }
    registerCreateFromCurrentProfileAction() {
        const that = this;
        this._register(registerAction2(class CreateFromCurrentProfileAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.profiles.actions.createFromCurrentProfile',
                    title: ( localize2WithPath(_moduleId, 24, "Save Current Profile As...")),
                    category: PROFILES_CATEGORY,
                    f1: true,
                    precondition: PROFILES_ENABLEMENT_CONTEXT
                });
            }
            run(accessor) {
                return that.userDataProfileImportExportService.createProfile(that.userDataProfileService.currentProfile);
            }
        }));
    }
    registerCreateProfileAction() {
        const that = this;
        this._register(registerAction2(class CreateProfileAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.profiles.actions.createProfile',
                    title: ( localize2WithPath(_moduleId, 25, "Create Profile...")),
                    category: PROFILES_CATEGORY,
                    precondition: PROFILES_ENABLEMENT_CONTEXT,
                    f1: true,
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '3_manage_profiles',
                            when: ( (ContextKeyExpr.and(
                                PROFILES_ENABLEMENT_CONTEXT,
                                 (CONTEXT_ENABLE_NEW_PROFILES_UI.negate())
                            ))),
                            order: 1
                        }
                    ]
                });
            }
            async run(accessor) {
                if (that.isNewProfilesUIEnabled()) {
                    return that.openProfilesEditor();
                }
                else {
                    return that.userDataProfileImportExportService.createProfile();
                }
            }
        }));
    }
    registerDeleteProfileAction() {
        this._register(registerAction2(class DeleteProfileAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.profiles.actions.deleteProfile',
                    title: ( localize2WithPath(_moduleId, 26, "Delete Profile...")),
                    category: PROFILES_CATEGORY,
                    f1: true,
                    precondition: ( (ContextKeyExpr.and(PROFILES_ENABLEMENT_CONTEXT, HAS_PROFILES_CONTEXT))),
                    menu: [
                        {
                            id: ProfilesMenu,
                            group: '3_manage_profiles',
                            when: ( (ContextKeyExpr.and(
                                PROFILES_ENABLEMENT_CONTEXT,
                                 (CONTEXT_ENABLE_NEW_PROFILES_UI.negate())
                            ))),
                            order: 2
                        }
                    ]
                });
            }
            async run(accessor) {
                const quickInputService = accessor.get(IQuickInputService);
                const userDataProfileService = accessor.get(IUserDataProfileService);
                const userDataProfilesService = accessor.get(IUserDataProfilesService);
                const userDataProfileManagementService = accessor.get(IUserDataProfileManagementService);
                const notificationService = accessor.get(INotificationService);
                const profiles = userDataProfilesService.profiles.filter(p => !p.isDefault && !p.isTransient);
                if (profiles.length) {
                    const picks = await quickInputService.pick(( (profiles.map(profile => ({
                        label: profile.name,
                        description: profile.id === userDataProfileService.currentProfile.id ? ( localizeWithPath(_moduleId, 27, "Current")) : undefined,
                        profile
                    })))), {
                        title: ( localizeWithPath(_moduleId, 28, "Delete Profile...")),
                        placeHolder: ( localizeWithPath(_moduleId, 29, "Select Profiles to Delete")),
                        canPickMany: true
                    });
                    if (picks) {
                        try {
                            await Promise.all(( (picks.map(pick => userDataProfileManagementService.removeProfile(pick.profile)))));
                        }
                        catch (error) {
                            notificationService.error(error);
                        }
                    }
                }
            }
        }));
    }
    registerHelpAction() {
        this._register(registerAction2(class HelpAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.profiles.actions.help',
                    title: PROFILES_TITLE,
                    category: Categories.Help,
                    menu: [{
                            id: MenuId.CommandPalette,
                        }],
                });
            }
            run(accessor) {
                return accessor.get(IOpenerService).open(( (URI.parse('https://aka.ms/vscode-profiles-help'))));
            }
        }));
    }
    async getProfileTemplatesQuickPickItems() {
        const quickPickItems = [];
        const profileTemplates = await this.userDataProfileManagementService.getBuiltinProfileTemplates();
        for (const template of profileTemplates) {
            quickPickItems.push({
                label: template.name,
                ...template
            });
        }
        return quickPickItems;
    }
    async reportWorkspaceProfileInfo() {
        await this.lifecycleService.when(LifecyclePhase.Eventually);
        const workspaceId = await this.workspaceTagsService.getTelemetryWorkspaceId(this.workspaceContextService.getWorkspace(), this.workspaceContextService.getWorkbenchState());
        this.telemetryService.publicLog2('workspaceProfileInfo', {
            workspaceId,
            defaultProfile: this.userDataProfileService.currentProfile.isDefault
        });
    }
};
UserDataProfilesWorkbenchContribution = ( (__decorate([
    ( (__param(0, IUserDataProfileService))),
    ( (__param(1, IUserDataProfilesService))),
    ( (__param(2, IUserDataProfileManagementService))),
    ( (__param(3, IUserDataProfileImportExportService))),
    ( (__param(4, ITelemetryService))),
    ( (__param(5, IWorkspaceContextService))),
    ( (__param(6, IWorkspaceTagsService))),
    ( (__param(7, IContextKeyService))),
    ( (__param(8, IConfigurationService))),
    ( (__param(9, IEditorGroupsService))),
    ( (__param(10, IInstantiationService))),
    ( (__param(11, IProductService))),
    ( (__param(12, ILifecycleService)))
], UserDataProfilesWorkbenchContribution)));

export { OpenProfileMenu, UserDataProfilesWorkbenchContribution };
